let C2C_Sites = [];

//update new and number of cats:
const updater = () => {
    GetAndUpdateNumOfCats(); //getting current number of cats
    createUID(); //setting uid or creating new one //todo: use also localStorage
    setExtensionState();
    getSiteRules(); //overrides the local one
};

const getSiteRules = function () {
    const script = document.createElement('script');
    script.src = "https://s3-us-west-2.amazonaws.com/commentstocats/sites/sites.js";
    document.body.appendChild(script);
    setTimeout(() => {
        console.log(C2C_Sites.length, "site rules in use");
    },1000);
};

//create or get uid for user, updating cookie and storage for content scripts:
const createUID = (callback) => {

    callback = callback || (() => {
    });

    chrome.storage.sync.get('UniqueID', function (data) {
        if (data.UniqueID) {
            console.log("active user:", data.UniqueID);
            callback(undefined, data.UniqueID); //returning the userid from the chrome db
            $.get('http://www.commentstocats.com/rateacat/syncuid.php?uid=' + data.UniqueID);
        } else {

            $.ajax({
                url: 'http://www.commentstocats.com/rateacat/syncuid.php',
                type: 'GET',
                success: function (data) {
                    let obj = JSON.parse(data);
                    if (obj.status == 'failed') {
                        callback(obj.error);
                    }
                    else {
                        //adding new userid
                        console.log("new user:", obj.uid);
                        chrome.storage.sync.set({'UniqueID': obj.uid}, function () {
                            callback(undefined, obj.uid);
                        });
                    }
                },
                error: function (error) {
                    callback(error);
                }
            });
        }
    })
};

//number of cats into local param to be used on content scripts:
const GetAndUpdateNumOfCats = (callback) => {

    callback = callback || (() => {
    });

    const lastCount = 171; //fallback, version set

    $.ajax({
        url: 'http://s3-us-west-2.amazonaws.com/commentstocats/cats.txt',
        type: 'GET',
        success: (result) => {
            console.log("current count:", result);
            chrome.storage.sync.set({'cats': result}, () => {
                callback(result);
            });
        },
        error: (error) => {
            callback(lastCount);
        }
    });
};

//setting extension state for usage in popup and content script:
const setExtensionState = () => {
    chrome.storage.sync.get('state', function (result) {
        if (typeof result.state != 'undefined' && !result.state) {
            chrome.browserAction.setIcon({path: {19: "../images/main_logo_trans_19_bw.png"}});
        }
        else if (result.state) {
            //nothing for now, keeping as true
        }
        else {
            chrome.storage.sync.set({'state': true}); //first time
        }
    });
};

//wrapping extension state getter as we use it multiple times:
const getExtensionState = (callback) => {
    chrome.storage.sync.get('state', (result) => {
        callback(result.state);
    });
};

//tracking (with sampling) for the supported or not, for knowing which sites to add next:
const trackSupported = (location) => {
    if (location.pathname == '/sharer/sharer.php')
        _gaq.push(['_trackEvent', 'contentScripts', 'share_cat', location.href.split('cat%3D')[1]]);
    else if (Math.random() > 0.9 && location.hostname != 'analytics.google.com')
        _gaq.push(['_trackEvent', 'visits', status ? 'supported' : 'unsupported', location.hostname]);
};

//checking if the url is activated or not:
const checkActivated = (match, callback) => {
    let domain = match.domain;
    chrome.storage.sync.get(domain, function (result) {
        if (typeof result[domain] == 'undefined') {
            return callback(true);
        }
        else if (result[domain] == 'disabled') { //exists and paused
            return callback(false);
        }
        else {
            //nothing
        }
    });
};

//checking if the url is in our supported urls list:
const checkSupportedURL = (url, callback) => {
    let l = document.createElement("a");
    l.href = url; //parsing url
    for (let i = 0; i < C2C_Sites.length; i++) { //going over array of sites and handling it accordingly
        if (C2C_Sites[i].domainRegex) { //regex match
            let regexCheck = new RegExp(C2C_Sites[i].domainRegex, 'g');
            if (l.href.match(regexCheck)) {
                return callback(true, l, i);
            }
        }
        else if (l.hostname.indexOf(C2C_Sites[i].domain) > -1) { //regular match
            return callback(true, l, i);
        }
    }
    callback(false, l); //when couldn't find anything
};

//in use for updating the current popup icon, doing it out of "checkSite" as we go between tabs and content is being called once
const setTabVisit = () => {
    getExtensionState((active) => {
        if (!active) //won't run anything if not active
            return;
        chrome.tabs.query({active: true, currentWindow: true}, function (tab) {
            if (!tab || !tab[0] || !tab[0].url)
                return;

            checkSupportedURL(tab[0].url, function (status, location, match) {
                trackSupported(location);
                if (status) {
                    checkActivated(C2C_Sites[match], function (activation) {
                        if (activation) {
                            chrome.browserAction.setBadgeText({text: ':)'}); //activated
                        } else {
                            chrome.browserAction.setBadgeText({text: ':('}); //not activated
                        }
                    });
                } else {
                    chrome.browserAction.setBadgeText({text: ''}); //not supported, so just nothing
                }
            });
        });
    });
};

//checking the current state and site, as asked from the content script
const checkSite = (url, sendResponse) => {
    checkSupportedURL(url, function (status, l, match) {
        if (status) { //this url is supported
            checkActivated(C2C_Sites[match], function (activation) {
                if (activation) {
                    sendResponse({shouldHide: true, rules: C2C_Sites[match]});  //this site activated right now, passing its settings
                } else {
                    sendResponse({shouldHide: false});  //this site is not activated right now
                }
            });
        } else {
            sendResponse({shouldHide: false}); //this site is not supported by our system
        }
    });
};
