const logger = (name, value) => {
    if (!name || !value)
        return console.error("missing name or value");

    chrome.runtime.sendMessage({event: 'logs', name: name, value: value});
};

const unDoneSelectors = (domainSettings) => {
    domainSettings.selectors.forEach((s) => {
        s.done = false;
    });
};

const findNodeToInjectCatTo = (domainSettings) => {
    let replaceField;
    if (Array.isArray(domainSettings.replaceField)) {
        for (let i = 0; i < domainSettings.replaceField.length; i++) {
            replaceField = $(domainSettings.replaceField[i].selectorSpecial ? domainSettings.replaceField[i].selectorSpecial() : domainSettings.replaceField[i].selector);
            if (replaceField[0])
                break;
        }
    } else {
        replaceField = $(domainSettings.replaceField.selectorSpecial ? domainSettings.replaceField.selectorSpecial() : domainSettings.replaceField.selector);
    }

    return replaceField;

};

const getNumberOfCats = (callback) => {
    chrome.storage.sync.get('cats', (result) => {
        if (!result.cats) { //couldn't find count, will try to reFetch it from background
            chrome.runtime.sendMessage({
                event: "ReRunNumOfCats"
            }, (result) => {
                try {
                    let cats = JSON.parse(result);
                    callback(null, cats);
                } catch (err) {
                    callback(err);
                }
            });
        }
        else { //got number of cats
            try {
                let cats = JSON.parse(result.cats);
                callback(null, cats);
            } catch (err) {
                callback(err);
            }
        }
    });
};

const getUniqueId = (callback) => {
    chrome.storage.sync.get('UniqueID', (result) => {
        if (!result.UniqueID) {
            chrome.runtime.sendMessage({
                event: "ReRunUniqueID"
            }, (response) => {
                callback(null, response.data || 'undefined');
            });
        }
        else {
            callback(null, result.UniqueID);
        }

    });
};

const getCatData = (catsCount, uid, callback) => {

    let max = catsCount.num;
    let randomCat = Math.floor(Math.random() * max + 1);
    logger("randomCat", randomCat.toString());

    chrome.runtime.sendMessage({
        event: "getCat",
        catid: randomCat,
        uid: uid || 'undefined'
    }, (response) => {

        let data = {
            catId: randomCat,
            error: (!response || response.data.status != "success"),
            status: (typeof error != 'undefined' || response.data.voted == 'notreal') ? ':(' : (response.data.voted ? '&#10003;' : '+'),
            statusClass: (typeof error != 'undefined' || response.data.voted == 'notreal') ? 'c2c_failed' : (response.data.voted ? 'c2c_added' : 'c2c_adder'),
            voter: (typeof error != 'undefined' && !response.data.voters) ? ':(' : response.data.voters
        };

        callback(null, data);

    });

};

const handleHTML = (replaceField, catsCount, data, callback) => {

    let catImg = chrome.extension.getURL("./images/main_logo_trans_128.png");
    let currentCat = "//s3-us-west-2.amazonaws.com/commentstocats/photos/Cat" + data.catId + ".jpg";
    let catInner = chrome.extension.getURL("./contentScripts/resources/catInner.html");

    if ($("#catAcross")[0])
        return;

    $.get(catInner, (response) => {

        $(response).insertBefore(replaceField);

        chrome.storage.sync.get('shown', (result) => {

            if (result.shown && result.shown === -1)
                return;

            if ((result.shown > 0 && result.shown % 100 === 0)) {

                $("<div class='c2c_msg'>" +
                    "Do you LOVE Comments to Cats?<br>" +
                    "<img src='" + catImg + "' style='width:50px;'>" +
                    "<br>" +
                    "Please " +
                    "<a class='storeReview' href='https://chrome.google.com/webstore/detail/comments-to-cats/hgjfibdfedjcopejmhnhcimefdlfdpig/reviews' target='_blank'>RATE US</a>" +
                    " or " +
                    "<a class='fbShare' href='http://www.facebook.com/sharer/sharer.php?u=https://goo.gl/yPh5om' target='_blank'>SHARE IT</a> " +
                    "with your friends!<br>" +
                    "Thanks!" +
                    "<br>" +
                    "<span class='hideShown StopShown small_c2c'>Don't show this again</span>" +
                    "<br>" +
                    "<span class='hideShown small_c2c'>Remind me later</span>" +
                    "</div>").insertBefore($(".catRate"));

                $(".storeReview").click(() => {
                    console.log("here");
                    logger("sharePopup", 'storeReview', result.shown);
                });

                $(".fbShare").click(() => {
                    logger("sharePopup", 'fbShare', result.shown);
                });

                $(".hideShown").click(() => {
                    logger("sharePopup", 'hide', result.shown);
                    $(".c2c_msg").remove();
                });

                $(".StopShown").click(() => {
                    logger("sharePopup", 'stop', result.shown);
                    chrome.storage.sync.set({shown: -1}, () => {
                    });
                });
            }

            chrome.storage.sync.set({shown: (result.shown ? ++result.shown : 1)}, () => {
            });

        });

        if (data.error)
            $(".catInfo")[0].innerHTML = "Error while handling cat's data, sorry.";

        $(".logocat").css('background-image', 'url(' + catImg + ')');
        if (data.catId >= catsCount.new)
            $("#c2c_ribbonWrapper").show();

        $("#catimg")
            .attr('src', currentCat)
            .attr('title', 'Meow to meet you, I am cat #' + data.catId + '!');

        $("#c2c_vote").text(data.voter);
        $("#voteTool")
            .addClass(data.statusClass)
            .html(data.status);

        callback();

    });

};

const setClicks = (domainSettings, uid, data, callback) => {

    //on cat major image events
    let catHover;
    $("#catimg")
        .click(() => {
            logger("catImg", 'click');
            let win = window.open('https://www.facebook.com/CommentsToCats', '_blank');
            win.focus();
        })
        .hover(() => {
            catHover = setTimeout(() => {
                logger("majorCatImg", 'hover');
            }, 200);
        }, () => {
            clearTimeout(catHover);
        });

    //on logo mini events
    let logocatHover;
    $(".logocat")
        .hover((e) => { //in
            $(e.target).addClass('c2cImgSpin');
            logocatHover = setTimeout(() => {
                logger("logoCatImg", 'hover');
            }, 200);
        }, (e) => { //out
            clearTimeout(logocatHover);
            $(e.target).removeClass('c2cImgSpin');
        });

    //show comments and hide cat
    $('#c2c_showComments').click(() => {
        logger("showComments", location.hostname);
        $("#showedCatContainer").hide();
        $("#hiddenCatContainer").addClass("displayHidden");

        unDoneSelectors(domainSettings);
        handleComments(domainSettings, false);

    });

    //show cat and hide comments
    $("#reshowCat").click(() => {
        logger("reShowComments", location.hostname);
        $("#hiddenCatContainer").removeClass("displayHidden");
        $("#hiddenCatContainer").removeClass("displayHidden");
        unDoneSelectors(domainSettings);
        handleComments(domainSettings, true);
        $("#showedCatContainer").show();
    });

    //on voting:
    $('.c2c_adder').click(() => {
        chrome.runtime.sendMessage({
            event: "voteCat",
            catid: data.catId,
            uid: uid
        }, (response) => {

            $('.c2c_adder').unbind('click').removeClass('c2c_adder').addClass('c2c_added');

            if (response.status == 'success') {
                $(".statusTask")[0].innerHTML = '&#10003;';
                let c2c_vote = $("#c2c_vote");
                c2c_vote[0].innerText++;

                logger("voted", domainSettings.domain);

            }
            else {
                $(".statusTask").innerText = ':(';
                $(".catInfo")[0].innerHTML = "Error while voting, sorry.";
                logger("voted_error", domainSettings.domain + "_" + data.catId + "_" + uid);
            }
        });
    });

    $("#c2c_fbshare").click(() => {
        window.open('http://www.facebook.com/sharer/sharer.php?u=http%3A%2F%2Fcommentstocats.com%2Fshare.php%3Fcat%3D' + data.catId, 'SHARE Comments to Cats', "height=200,width=600");
    });

    callback();

};

const injectCat = (domainSettings, finishCallback) => {

    //finding the relevant field to replace and inject to:

    const replaceField = findNodeToInjectCatTo(domainSettings);
    if (!replaceField[0]) {
        finishCallback("no selector to replace"); //no place to inject cat into
    }
    else {
        try {

            let catsCount, uniqueId, catData;
            async.waterfall([
                    /*getNumberOfCats*/(next) => {
                        getNumberOfCats((err, response) => {
                            if (err)
                                next(err);
                            else {
                                catsCount = response;
                                next();
                            }
                        });
                    },
                    /*GetUID*/(next) => {
                        getUniqueId((err, response) => {
                            if (err)
                                next(err);
                            else {
                                uniqueId = response;
                                next();
                            }
                        });
                    },
                    /*chooseCatAndGetData*/(next) => {
                        getCatData(catsCount, uniqueId, (err, response) => {
                            if (err)
                                next(err);
                            else {
                                catData = response;
                                next();
                            }
                        });
                    },
                    /*handleHTML*/(next) => {
                        handleHTML(replaceField, catsCount, catData, next);
                    },
                    /*setClicks*/(next) => {
                        setClicks(domainSettings, uniqueId, catData, next);
                    }
                ], /*done*/
                (err) => {
                    if (err)
                        return finishCallback(err);

                    finishCallback();

                }
            );
        }
        catch (error) {
            finishCallback(error);
        }
    }
};

const handleSelectors = (settings, hide, callback) => {

    let selectorHandler = (item, cb) => {
        if (item.done) {
            return cb();
        }

        try {

            let selected = $(item.selectorSpecial ? item.selectorSpecial() : item.selector);
            if (!selected[0]) {
                return cb();
            }

            for (let i = 0; i < selected.length; i++) { //selector can handle multiple elements on page (try not to though due async page creation)
                let currentSelector = $(selected[i]);
                if (!hide) {
                    showComments = true;
                    currentSelector.css('display', 'block');
                    currentSelector.css('visibility', 'visible');
                } else {
                    currentSelector.css('display', 'none');
                    currentSelector.css('visibility', 'hidden');

                    let observer = new MutationObserver((mutations) => { //when styling changes and not asked for it, keep it (disable site side override and update)
                        mutations.forEach((mutationRecord) => {
                            if (!showComments) {
                                currentSelector.css('display', 'none');
                                currentSelector.css('visibility', 'hidden');
                            }
                        });
                    });
                    observer.observe(currentSelector[0], {attributes: true, attributeFilter: ['style']});
                }
            }
            item.done = true;
            cb(null, item);

        }
        catch (e) {
            cb(e);
        }
    };

    async.map(settings.selectors, selectorHandler, (err, results) => {
        if (err)
            return callback(err);

        if (results.every((selector) => { //if done all - > callback
                return selector ? selector.done : false; //done all selectors?
            })) {
            return callback(null, true);
        } else {
            callback(); //if not all selectors done we'll retry until they are/timeout kills is all
        }
    });
};

const handleComments = (settings, hide, callback) => {

    callback = callback || (() => {
    });

    if (hide) { //setting interval to find elements to hide, the page is async so we'll try for some time
        let timeout;
        let interval = setInterval(() => {
            handleSelectors(settings, hide, (err, done) => {
                if (err) {
                    //future use
                } else if (done) {
                    clearInterval(interval);
                    if (timeout)
                        clearTimeout(timeout);
                    callback();
                } else {
                    //nothing for now, will just try again until timeout close it
                }
            });
        }, intervalChecks);

        timeout = setTimeout(() => { //stop trying and go to cat injection automatically
            clearInterval(interval);
            callback();
        }, timeoutSelectors);

    } else { //no need for interval or timeot, they are on the page...
        handleSelectors(settings, hide, (response) => {
            //console.log("done showing");
        });
    }

};
